"""
pygrnwang
==========
This Python package serves as the frontend for calculating and building a Green's function library
for synthetic seismograms. The backend consists of Wang Rongjiang's program for calculating synthetic
seismograms in layered Earth models, including EDGRN/EDCMP QSEIS, SPGRN, and QSSP (Wang, 1999;
Wang et al., 2017).

"""

__version__ = "1.0.0"
